// gdith.c --
//    Procedures dealing with grey-scale and mono dithering, 
//    as well as X Windows set up procedures.
#include <math.h>
#include "video.h"
#include "proto.h"
#include "dither.h"
//#include <sys/time.h>

#include <stdlib.h>
#include <string.h>

#define AMIGA


// Range values for lum, cr, cb.
int LUM_RANGE;
int CR_RANGE;
int CB_RANGE;

// Array that remaps color numbers to actual pixel values used by X server.
unsigned char pixel[256];
unsigned long wpixel[256];

// Arrays holding quantized value ranged for lum, cr, and cb. 
int *lum_values;
int *cr_values;
int *cb_values;

// Declaration of global variable containing dither type.

// Frame Rate Info
extern int framerate;

// Video rates table
// Cheat on Vid rates, round to 30, and use 30 if illegal value Except for 9, 
// where Xing means 15, and given their popularity, we'll be nice and do it
static int VidRateNum[16]=
{
  30, 24, 24, 25, 30, 30, 50, 60, 
  60, 15, 30, 30, 30, 30, 30, 30
};

void CloseXDisplay(void)
{
  /*if (CyberGfxBase && AslBase) CloseLibraries();
  if (screen) CloseCGXScreen();
  */
  printf("Closing Display...\n");
}

// ConvertColor -- Given a l, cr, cb tuple, converts it to r,g,b.
// Results:      r,g,b values returned in pointers passed as parameters.
// Side effects: None.
static void ConvertColor(unsigned int l, unsigned int cr, unsigned int cb, unsigned char *r, unsigned char *g, unsigned char *b)
{
  double fl, fcr, fcb, fr, fg, fb;

  fl = 1.164*(((double) l)-16.0);
  fcr =  ((double) cr) - 128.0;
  fcb =  ((double) cb) - 128.0;
  fr = fl + (1.366 * fcr);
  fg = fl - (0.700 * fcr) - (0.334 * fcb);
  fb = fl + (1.732 * fcb);
 
  *r = (unsigned char) fr;
  *g = (unsigned char) fg;
  *b = (unsigned char) fb;
}

// InitColorDisplay -- Initialized display for full color output.
// Results:      None.
// Side effects: None.


void InitColorDisplay(XInfo *xinfo )
{
  printf("Initialising ColorDisplay...\n");
  /*if (OpenLibraries())
    if(OpenCGXScreen(0,FALSE))
      printf("Display inititalised...\n");
    else
      exit(-1);
  else
    exit(-1);
  */

  if ((xinfo->ditherType == NO_DITHER) || (xinfo->ditherType == PPM_DITHER))
     return;

  if ((noDisplayFlag) || (xinfo==NULL)) return;
  
  //if (xinfo->ditherType == FULL_COLOR_DITHER)
  if (!noDisplayFlag)
    printf("Creating window...\n");
}

// ExecuteDisplay -- Actually displays display plane in previously created window.
// Results:       None.
// Side effects:  None.
void ExecuteDisplay(VidStream *vid_stream, XInfo *xinfo)
{
  static int rate_deal = -1;
  static int one_frame_time;
  //static struct timeval tftarget, tfnow;
  //register int usec, sec;

  vid_stream->totNumFrames++;

  if (partialFlag) 
  {
    if ((endFrame != -1) && (vid_stream->totNumFrames > endFrame)) 
    {
      PrintTimeInfo(vid_stream);

      vid_stream->film_has_ended=TRUE;
      if (loopFlag)
        clear_data_stream(vid_stream);
      else
        DestroyVidStream(vid_stream, xinfo);

      return;
    }
    if (vid_stream->totNumFrames < startFrame) {
      return;
    }
  }

  // hier switch frameratecontrol einfgen....

  if (vid_stream->ditherType == NO_DITHER) return;
  if (vid_stream->ditherType == PPM_DITHER) 
  {
    ExecutePPM(vid_stream);
    return;
  }

  if (xinfo==NULL) return;
}

#define PPM_BITS 8
// ExecutePPM -- Write out a display plane as a PPM file.
// Results:      None.
// Side effects: None

#ifndef AMIGA

void ExecutePPM(VidStream *vid_stream)
{
  static int munged = 0;
  static char mungedInputName[300];
  char fileName[300];
  unsigned int *p;
  unsigned int r, g, b;

  static int count=0;

int n;
FILE *file;

  p = (unsigned int *) vid_stream->current->display;

  n = vid_stream->h_size * vid_stream->v_size;

  sprintf(fileName, "Bild_%05d.ppm", count++);
  if ( (file = fopen(fileName, "w")) )
  {
    fprintf(file, "P6\n%d %d\n255\n", vid_stream->h_size, vid_stream->v_size);

    while (n > 0) 
    {
      r = *p & 0xff;
      g = (*p >> PPM_BITS) & 0xff;
      b = (*p >> (2*PPM_BITS)) & 0xff;
     
      putc(r, file);
      putc(g, file);
      putc(b, file);

      ++p;
      --n;
    }
    fclose(file);
  }
  else
    printf("Fehler beim ffnen der Ausgabedatei %s !!\n",fileName);
}

#endif





